"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const status_1 = require("./errors/status");
const methods_1 = require("./methods");
const route_1 = require("./route");
const utils_1 = require("./utils");
class Router {
    constructor() {
        this.routes = [];
        this.middlewares = [];
        this.child = (path, handler, options) => {
            let router;
            let routerMiddlewares;
            if (handler instanceof Router) {
                router = handler;
            }
            else if (Array.isArray(handler)) {
                router = handler[0];
                routerMiddlewares = utils_1.toArray(handler[1]);
            }
            if (!router) {
                router = new Router();
            }
            if (routerMiddlewares) {
                router.middleware(routerMiddlewares);
            }
            this.routes.push(new route_1.Route(undefined, path, router, options));
            return router;
        };
        this.route = (method, path, handler, options) => {
            this.routes.push(new route_1.Route(utils_1.toLowerCases(method), path, handler, Object.assign({ exact: true }, options)));
            return this;
        };
        this.any = (path, handler, options) => {
            return this.route(undefined, path, handler, options);
        };
        this.middleware = (middleware) => {
            if (middleware) {
                this.middlewares.push(...utils_1.toArray(middleware));
            }
            return this;
        };
        this.handle = (ctx) => __awaiter(this, void 0, void 0, function* () {
            try {
                const middlewaresNext = [];
                for (const middleware of this.middlewares) {
                    const next = yield middleware(ctx);
                    if (next) {
                        middlewaresNext.push(next);
                    }
                }
                if (!ctx.req.method || !ctx.req.url) {
                    /* istanbul ignore next */
                    // Invalid HTTP request, should never happen
                    throw new status_1.ErrorWithStatusCode(400, "Invalid request");
                }
                if (!ctx.path || !ctx.path.startsWith("/")) {
                    ctx.path = "/" + (ctx.path || "");
                }
                const method = ctx.req.method.toLowerCase();
                const url = ctx.path;
                let route;
                let match;
                const params = {};
                const optionsAllowedMethods = [];
                this.routes.some(testRoute => {
                    if (testRoute.regex) {
                        match = testRoute.regex.exec(url);
                        if (!match) {
                            return false;
                        }
                        else {
                            // Lots of ifs, but need to be explicit for TypeScript
                            if (testRoute.method) {
                                if (Array.isArray(testRoute.method)) {
                                    optionsAllowedMethods.push(...testRoute.method);
                                }
                                else {
                                    optionsAllowedMethods.push(testRoute.method);
                                }
                            }
                            else {
                                optionsAllowedMethods.push(...methods_1.allMethods);
                            }
                        }
                    }
                    const matchesMethod = testRoute.method &&
                        testRoute.method !== method &&
                        !(Array.isArray(testRoute.method) && testRoute.method.includes(method));
                    if (matchesMethod) {
                        return false;
                    }
                    if (match) {
                        ctx.path = ctx.path.replace(match[0], "");
                    }
                    route = testRoute;
                    return true;
                });
                if (route && match) {
                    function applyNextMiddlewares() {
                        return __awaiter(this, void 0, void 0, function* () {
                            for (const middelwareNext of middlewaresNext) {
                                yield middelwareNext();
                            }
                        });
                    }
                    try {
                        for (let index = 1; index < match.length; index++) {
                            const key = route.keys[index - 1];
                            const prop = key.name;
                            const value = utils_1.decodeParam(match[index]);
                            if (value !== undefined && !(prop in params)) {
                                params[prop] = value;
                            }
                        }
                        ctx.params = Object.assign({}, ctx.params, params);
                        yield route.handle(ctx);
                    }
                    catch (error) {
                        yield applyNextMiddlewares();
                        throw error;
                    }
                    yield applyNextMiddlewares();
                    return;
                }
                if (method === methods_1.Methods.OPTIONS) {
                    // Prevent duplicates methods, all upper-case
                    const methods = [];
                    optionsAllowedMethods.forEach(optionsMethod => {
                        if (methods.includes(optionsMethod.toUpperCase())) {
                            return;
                        }
                        methods.push(optionsMethod.toUpperCase());
                    });
                    ctx.res.setHeader("Allow", methods.join(", "));
                    return;
                }
                // 404
                throw new status_1.ErrorWithStatusCode(404, "Not Found");
            }
            catch (error) {
                if (this.errorHandler) {
                    return this.errorHandler(ctx, error);
                }
                throw error;
            }
        });
    }
}
exports.Router = Router;
// Attach method shortcuts such as router.get(...), etc
Object.values(methods_1.Methods).forEach(method => {
    // @ts-ignore
    Router.prototype[method] = function (path, handler, options) {
        return this.route(method, path, handler, options);
    };
});
//# sourceMappingURL=router.js.map