"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const parseUrl = require("parseurl");
const qs = require("qs");
const throng = require("throng");
const uuid = require("uuid");
const defaultHandler_1 = require("./errors/defaultHandler");
const router_1 = require("./router");
const utils_1 = require("./utils");
class Routex extends router_1.Router {
    constructor({ requestId, errorHandler, providers } = {}) {
        super();
        this.errorHandler = defaultHandler_1.defaultErrorHandler;
        this.appMiddlewares = [];
        /**
         * Adds app middleware(s)
         */
        this.appMiddleware = (appMiddleware) => {
            if (appMiddleware) {
                this.appMiddlewares.push(...utils_1.toArray(appMiddleware).map(appMiddlewareOne => appMiddlewareOne(this)));
            }
            return this;
        };
        /**
         * Handles HTTP request
         */
        this.handler = (req, res) => __awaiter(this, void 0, void 0, function* () {
            // Parse query string and extract path
            const parsed = parseUrl(req);
            const query = parsed &&
                (utils_1.isString(parsed.query)
                    ? qs.parse(parsed.query)
                    : parsed.query);
            /* istanbul ignore next */
            const ctx = {
                data: {},
                params: {},
                path: (parsed && parsed.pathname) || "/",
                query: query || {},
                req,
                requestId: this.requestIdGenerator
                    ? this.requestIdGenerator()
                    : undefined,
                res,
                workerId: this.workerId,
                providers: Object.assign({}, this.providers)
            };
            yield this.handle(ctx);
            if (ctx.body) {
                ctx.res.setHeader("Content-Length", ctx.body.contentLength);
                ctx.res.setHeader("Content-Type", ctx.body.contentType);
            }
            if (ctx.statusCode) {
                ctx.res.writeHead(ctx.statusCode);
            }
            if (ctx.body) {
                ctx.body.write(ctx.res);
            }
            ctx.res.end();
            return ctx;
        });
        if (requestId !== false) {
            this.requestIdGenerator = requestId || (() => uuid());
        }
        if (errorHandler) {
            this.errorHandler = errorHandler;
        }
        this.providers = providers || {};
    }
    /**
     * Starts the server
     */
    listen(port, { hostname, server: optionsServer, https: httpsOptions } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (port) {
                const originalPort = port;
                if (utils_1.isString(port)) {
                    port = parseInt(port, 10);
                }
                if (!Number.isInteger(port)) {
                    throw new TypeError(`Invalid port (${originalPort} is not integer/string).`);
                }
            }
            if (hostname && !utils_1.isString(hostname)) {
                throw new TypeError(`Invalid hostname (${hostname} is not a string)`);
            }
            const server = this.initializeServer(optionsServer ||
                (httpsOptions
                    ? https.createServer(httpsOptions, this.handler)
                    : http.createServer(this.handler)));
            yield new Promise(resolve => {
                server.listen({ port: port, host: hostname }, resolve);
            });
            const close = () => new Promise((resolve, reject) => {
                server.close(error => {
                    if (error) {
                        /* istanbul ignore next */
                        reject(error);
                    }
                    resolve();
                });
            });
            const address = server.address();
            return {
                close,
                port: 
                /* istanbul ignore next */
                address && !utils_1.isString(address) ? address.port : null,
                server
            };
        });
    }
    /* istanbul ignore next */
    cluster(port, _a = {}) {
        var { workers } = _a, options = __rest(_a, ["workers"]);
        throng({
            start: id => {
                this.workerId = id;
                this.listen(port, options);
            },
            workers
        });
    }
    /**
     * Decorates a server using app middlewares
     */
    initializeServer(server) {
        return this.appMiddlewares.reduce((reducingServer, appMiddleware) => (appMiddleware.initializeServer &&
            appMiddleware.initializeServer(reducingServer)) ||
            reducingServer, server);
    }
}
exports.Routex = Routex;
//# sourceMappingURL=routex.js.map