/// <reference types="node" />
import * as http from "http";
import * as https from "https";
import { AppMiddleware } from "./appMiddleware";
import { ICtx, ICtxProviders } from "./ctx";
import { ErrorHandler } from "./handler";
import { Router } from "./router";
export interface IListenOptions {
    hostname?: string;
    server?: http.Server;
    https?: https.ServerOptions;
    cluster?: boolean | number;
}
export interface IClusterOptions extends IListenOptions {
    workers?: number | string;
}
export interface IRoutexOptions {
    requestId?: (() => string) | false;
    errorHandler?: ErrorHandler;
    providers?: ICtxProviders;
}
export declare class Routex extends Router {
    requestIdGenerator?: () => string;
    errorHandler: ErrorHandler;
    private appMiddlewares;
    private workerId?;
    providers: ICtxProviders;
    constructor({ requestId, errorHandler, providers }?: IRoutexOptions);
    /**
     * Adds app middleware(s)
     */
    appMiddleware: (appMiddleware: AppMiddleware | AppMiddleware[]) => this;
    /**
     * Handles HTTP request
     */
    handler: (req: http.IncomingMessage, res: http.ServerResponse) => Promise<ICtx>;
    /**
     * Starts the server
     */
    listen(port?: number | string, { hostname, server: optionsServer, https: httpsOptions }?: IListenOptions): Promise<{
        close: () => Promise<unknown>;
        port: number | null;
        server: http.Server | https.Server;
    }>;
    cluster(port?: number | string, { workers, ...options }?: IClusterOptions): void;
    /**
     * Decorates a server using app middlewares
     */
    initializeServer(server: http.Server | https.Server): http.Server | https.Server;
}
